"""Library to handle connection with UberSmart."""
from __future__ import annotations

import logging
import struct
import time
from typing import Any

from .device import UberSolarBaseDevice

_LOGGER = logging.getLogger(__name__)


class UberSmart(UberSolarBaseDevice):
    """Representation of a UberSmart Device."""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """UberSolar UberSmart constructor."""

        super().__init__(*args, **kwargs)

    async def toggle_switches_all(self, switches: str) -> None:
        """Set all switches from hex string."""

        # 1st byte is message id, use 06 to toggle switches.
        # need to send  all switches.
        # elementToggle, pumpToggle, holidayModeToggle, solenoidMode = 0 - off, 1 - on, 2 - auto
        # example 0600000002
        if not len(switches) == 8:
            _LOGGER.error("Switch length has to be 8")

        await self._send_command(key=f"06{switches}")
        _LOGGER.info("%s: Toggle switches", self.name)

    async def enable_wifi_ap(self) -> None:
        """Enable Wifi ap mode on device."""
        # intended for firmware update but who knows.
        await self._send_command(key="14")
        _LOGGER.info("%s: Enable Wifi AP on device", self.name)

    async def set_current_time(self) -> None:
        """Set current datetime on device."""

        current_time = int(time.time())
        ct_to_bytearray = bytearray(struct.pack("<Qxxxx", current_time))
        ct_to_bytearray[9] = 2  # add utc offset on byte 9

        await self._send_command(key=f"09{ct_to_bytearray.hex()}")
        _LOGGER.info("%s: Send current time to device", self.name)

    async def turn_on_element(self) -> None:
        """Turn element switch on."""

        if not self.status_data:
            await self.update()

        current_switches = self.status_data[self._device.address]["AllSwitches"]

        if not len(current_switches) == 5:
            _LOGGER.error("Switch length has to be 5 bytes")

        current_switches[0] = 6
        current_switches[1] = 1

        await self._send_command(key=current_switches.hex())
        _LOGGER.info("%s: Turn Element On", self.name)

    async def turn_off_element(self) -> None:
        """Turn element switch off."""

        if not self.status_data:
            await self.update()

        current_switches = self.status_data[self._device.address]["AllSwitches"]

        if not len(current_switches) == 5:
            _LOGGER.error("Switch length has to be 5 bytes")

        current_switches[0] = 6
        current_switches[1] = 0

        await self._send_command(key=current_switches.hex())
        _LOGGER.info("%s: Turn Element Off", self.name)

    async def turn_on_pump(self) -> None:
        """Turn pump switch on."""

        if not self.status_data:
            await self.update()

        current_switches = self.status_data[self._device.address]["AllSwitches"]

        if not len(current_switches) == 5:
            _LOGGER.error("Switch length has to be 5 bytes")

        current_switches[0] = 6
        current_switches[2] = 1

        await self._send_command(key=current_switches.hex())
        _LOGGER.info("%s: Turn Pump On", self.name)

    async def turn_off_pump(self) -> None:
        """Turn pump switch off."""

        if not self.status_data:
            await self.update()

        current_switches = self.status_data[self._device.address]["AllSwitches"]

        if not len(current_switches) == 5:
            _LOGGER.error("Switch length has to be 5 bytes")

        current_switches[0] = 6
        current_switches[2] = 0

        await self._send_command(key=current_switches.hex())
        _LOGGER.info("%s: Turn Pump Off", self.name)

    async def turn_on_holiday(self) -> None:
        """Turn holiday switch on."""

        if not self.status_data:
            await self.update()

        current_switches = self.status_data[self._device.address]["AllSwitches"]

        if not len(current_switches) == 5:
            _LOGGER.error("Switch length has to be 5 bytes")

        current_switches[0] = 6
        current_switches[3] = 1

        await self._send_command(key=current_switches.hex())
        _LOGGER.info("%s: Turn Holiday On", self.name)

    async def turn_off_holiday(self) -> None:
        """Turn holiday switch off."""

        if not self.status_data:
            await self.update()

        current_switches = self.status_data[self._device.address]["AllSwitches"]

        if not len(current_switches) == 5:
            _LOGGER.error("Switch length has to be 5 bytes")

        current_switches[0] = 6
        current_switches[3] = 0

        await self._send_command(key=current_switches.hex())
        _LOGGER.info("%s: Turn Holiday Off", self.name)

    async def set_solinoid_off(self) -> None:
        """Turn Solinoid off."""

        if not self.status_data:
            await self.update()

        current_switches = self.status_data[self._device.address]["AllSwitches"]

        if not len(current_switches) == 5:
            _LOGGER.error("Switch length has to be 5 bytes")

        current_switches[0] = 6
        current_switches[4] = 0

        await self._send_command(key=current_switches.hex())
        _LOGGER.info("%s: Turn Solinoid Off", self.name)

    async def set_solinoid_on(self) -> None:
        """Turn Solinoid on."""

        if not self.status_data:
            await self.update()

        current_switches = self.status_data[self._device.address]["AllSwitches"]

        if not len(current_switches) == 5:
            _LOGGER.error("Switch length has to be 5 bytes")

        current_switches[0] = 6
        current_switches[4] = 1

        await self._send_command(key=current_switches.hex())
        _LOGGER.info("%s: Turn Solinoid On", self.name)

    async def set_solinoid_auto(self) -> None:
        """Set Solinoid to Auto."""

        if not self.status_data:
            await self.update()

        current_switches = self.status_data[self._device.address]["AllSwitches"]

        if not len(current_switches) == 5:
            _LOGGER.error("Switch length has to be 5 bytes")

        current_switches[0] = 6
        current_switches[4] = 2

        await self._send_command(key=current_switches.hex())
        _LOGGER.info("%s: Turn Solinoid to Auto", self.name)



"""Library to decode UberSmart Response."""
from __future__ import annotations

import datetime
import struct
from typing import Any


def process_ubersmart(data: bytearray) -> dict[str, Any]:
    """Process UberSmart data."""

    if data[0] == 1:
        return {
            "fWaterTemperature": round(struct.unpack("<f", data[1:5])[0], 2),
            "fManifoldTemperature": round(struct.unpack("<f", data[5:9])[0], 2),
            "fStoredWater": round(struct.unpack("<f", data[9:13])[0], 2),
        }

    if data[0] == 2:
        return {
            "bElementOn": data[1],
            "bPumpOn": data[2],
            "bHolidayMode": data[3],
            "eSolenoidMode": data[4],
            "fSolenoidState": round(struct.unpack("<f", data[5:9])[0], 2),
            "AllSwitches": data[:5],
        }

    if data[0] == 3:
        _llu_time = struct.unpack("<Q", data[1:9])[0]
        return {
            "lluTime": datetime.datetime.fromtimestamp(_llu_time).strftime(
                "%Y-%m-%d %H:%M:%S"
            ),
            "fHours": round(struct.unpack("<f", data[9:13])[0], 2),  # Time on
            "wLux": struct.unpack("<H", data[13:15])[0],
        }

    if data[0] == 4:
        return {
            "wRSSI": struct.unpack("<h", data[1:3])[0],
            "fPanelVoltage": round(struct.unpack("<f", data[3:7])[0], 2),
            "fChipTemp": round(struct.unpack("<f", data[7:11])[0], 2),
            "fWaterLevel": round(struct.unpack("<f", data[11:15])[0], 2),
            "fTankSize": round(struct.unpack("<f", data[15:19])[0], 2),
        }

    if data[0] == 5:
        return {
            "bPanelFaultCode": struct.unpack("<B", data[1:2])[0],
            "bElementFaultCode": struct.unpack("<B", data[2:3])[0],
            "bPumpFultCode": struct.unpack("<B", data[3:4])[0],
            "bSolenoidFaultCode": struct.unpack("<B", data[4:5])[0],
        }

    return {}
